@@ -29,17 +29,9 @@
 #include "gog-renderer.h"
 #include <goffice/data/go-data-impl.h>
 #include <goffice/data/go-data.h>
-#include <goffice/gtk/go-color-palette.h>
-#include <goffice/gtk/go-combo-color.h>
-#include <goffice/gtk/go-combo-pixmaps.h>
-#include <goffice/gtk/goffice-gtk.h>
 #include <goffice/utils/go-math.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <glade/glade-xml.h>
+
 #include <glib/gi18n-lib.h>
 
 #include <string.h>
@@ -49,14 +41,23 @@
 typedef GObjectClass GogErrorBarClass;
 static GObjectClass *error_bar_parent_klass;
 
-#ifdef WITH_GTK
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/go-color-palette.h>
+#include <goffice/gtk/go-combo-color.h>
+#include <goffice/gtk/go-combo-pixmaps.h>
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtklabel.h>
+#include <glade/glade-xml.h>
 typedef struct {
-	GogSeries *series;
-	GogErrorBar *bar;
-	char const* property;
-	GogErrorBarDisplay display;
-	GOColor color;
-	double width, line_width;
+	GogSeries 	   *series;
+	GogErrorBar 	   *bar;
+	char const 	   *property;
+	GogErrorBarDisplay  display;
+	GOColor 	    color;
+	double 		    width, line_width;
 } GogErrorBarEditor;
 
 static void
@@ -171,10 +172,10 @@ cb_type_changed (GtkWidget *w, GogErrorBarEditor *editor)
 
 gpointer
 gog_error_bar_prefs (GogSeries *series,
-			char const* property,
-			gboolean horizontal,
-			GogDataAllocator *dalloc,
-			GOCmdContext *cc)
+		     char const *property,
+		     gboolean horizontal,
+		     GogDataAllocator *dalloc,
+		     GOCmdContext *cc)
 {
 	GladeXML *gui;
 	GtkWidget *w, *bar_prefs;
@@ -183,9 +184,9 @@ gog_error_bar_prefs (GogSeries *series,
 	GogDataset *set;
 	GdkPixbuf *pixbuf;
 	GogErrorBarEditor *editor;
-	
+
 	g_return_val_if_fail (IS_GOG_SERIES (series), NULL);
-	
+
 	editor = g_new0 (GogErrorBarEditor, 1);
 	editor->series = series;
 	editor->property = property;
