@@ -40,7 +40,7 @@
 #include <libart_lgpl/art_render_gradient.h>
 #include <libart_lgpl/art_render_svp.h>
 #include <libart_lgpl/art_render_mask.h>
-#include <pango/pangoft2.h>
+#include <pango/pangocairo.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-output-stdio.h>
@@ -760,7 +760,9 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 					surface = cairo_pdf_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
+#endif
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo PDF backend missing");
@@ -771,7 +773,9 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 					surface = cairo_ps_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
+#endif
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo PS backend missing");
@@ -782,7 +786,9 @@ gog_renderer_cairo_export_image (GogRenderer *renderer, GOImageFormat format,
 					surface = cairo_svg_surface_create_for_stream 
 						(grc_cairo_write_func, 
 						 output, width_in_pts, height_in_pts);
+#ifdef HAVE_CAIRO_SURFACE_SET_FALLBACK_RESOLUTION
 					cairo_surface_set_fallback_resolution (surface, x_dpi, y_dpi);
+#endif
 					break;
 #else
 					g_warning ("[GogRendererCairo::export_image] cairo SVG backend missing");
