@@ -27,6 +27,15 @@
 #include <gsf/gsf-impl-utils.h>
 #include <glib/gi18n-lib.h>
 
+static GOImageFormatInfo *pixbuf_image_format_infos = NULL;
+static GHashTable *pixbuf_mimes = NULL;
+static unsigned pixbuf_format_nbr = 0;
+static gboolean pixbuf_format_done = FALSE;
+
+#define PIXBUF_IMAGE_FORMAT_OFFSET (1+GO_IMAGE_FORMAT_UNKNOWN)
+
+static void go_image_build_pixbuf_format_infos (void);
+
 /**
  * go_mime_to_image_format:
  * @mime_type: a mime type string
@@ -38,21 +47,23 @@ char *
 go_mime_to_image_format (char const *mime_type)
 {
  	guint i;
-	const char *suffix;
 	const char* exceptions[] = {
-		"svg+xml", "svg",
-		"x-wmf", "wmf",
-		"x-emf", "emf",
+		"image/svg", "svg",
+		"image/svg+xml", "svg",
+		"image/svg-xml", "svg",
+		"image/vnd.adobe.svg+xml", "svg",
+		"text/xml-svg", "svg",
+		"image/x-wmf", "wmf",
+		"image/x-emf", "emf",
 	};
 
-	if (strncmp (mime_type, "image/", 6) != 0)
-		return NULL;
-	suffix = mime_type + 6;
-	for (i = 0; i < G_N_ELEMENTS (exceptions); i +=2)
-		if (strcmp (suffix, exceptions[i]) == 0)
-			return g_strdup (exceptions[i+1]);
+	for (i = 0; i < G_N_ELEMENTS (exceptions); i += 2)
+		if (strcmp (mime_type, exceptions[i]) == 0)
+			return g_strdup (exceptions[i + 1]);
 
-	return g_strdup (suffix);
+	go_image_build_pixbuf_format_infos ();
+
+	return g_strdup (g_hash_table_lookup (pixbuf_mimes, mime_type));
 }
 
 /**
@@ -69,12 +80,8 @@ go_image_format_to_mime (char const *format)
  	guint i;
 #ifdef GOFFICE_WITH_GTK
 	GSList *ptr, *pixbuf_fmts;
-	GdkPixbufFormat *pfmt;
-	gchar *name;
-	int cmp;
-	gchar **mimes;
 #endif
-	const char* formats[] = {
+	static const char* const formats[] = {
 		"svg", "image/svg,image/svg+xml",
 		"wmf", "x-wmf",
 		"emf", "x-emf",
@@ -83,20 +90,21 @@ go_image_format_to_mime (char const *format)
 	if (format == NULL)
 		return NULL;
 
-	for (i = 0; i < G_N_ELEMENTS (formats); i +=2)
+	for (i = 0; i < G_N_ELEMENTS (formats); i += 2)
 		if (strcmp (format, formats[i]) == 0)
-			return g_strdup (formats[i+1]);
+			return g_strdup (formats[i + 1]);
 
 #ifdef GOFFICE_WITH_GTK
 	/* Not a format we have special knowledge about - ask gdk-pixbuf */
 	pixbuf_fmts = gdk_pixbuf_get_formats ();
 	for (ptr = pixbuf_fmts; ptr != NULL; ptr = ptr->next) {
-		pfmt = (GdkPixbufFormat *)ptr->data;
-		name = gdk_pixbuf_format_get_name (pfmt);
-		cmp = strcmp (format, name);
+		GdkPixbufFormat *pfmt = ptr->data;
+		gchar *name = gdk_pixbuf_format_get_name (pfmt);
+		int cmp = strcmp (format, name);
 		g_free (name);
 		if (cmp == 0) {
-			mimes = gdk_pixbuf_format_get_mime_types (pfmt);
+			gchar **mimes =
+				gdk_pixbuf_format_get_mime_types (pfmt);
 			ret = g_strjoinv (",", mimes);
 			g_strfreev (mimes);
 			break;
@@ -125,12 +133,6 @@ static GOImageFormatInfo const image_format_infos[GO_IMAGE_FORMAT_UNKNOWN] = {
 		(char *) "wmf", FALSE, FALSE, TRUE}
 };
 
-static GOImageFormatInfo *pixbuf_image_format_infos = NULL;
-static unsigned pixbuf_format_nbr = 0;
-static gboolean pixbuf_format_done = FALSE;
-
-#define PIXBUF_IMAGE_FORMAT_OFFSET (1+GO_IMAGE_FORMAT_UNKNOWN)
-
 static void
 go_image_build_pixbuf_format_infos (void)
 {
@@ -138,8 +140,8 @@ go_image_build_pixbuf_format_infos (void)
 	GdkPixbufFormat *fmt;
 	GSList *l, *pixbuf_fmts;
 	GOImageFormatInfo *format_info;
-	gchar **exts;
-	unsigned i;
+	gchar **exts, **mimes;
+	unsigned i, j;
 
 	if (pixbuf_format_done)
 		return;
@@ -149,6 +151,8 @@ go_image_build_pixbuf_format_infos (void)
 	
 	if (pixbuf_format_nbr > 0) {
 		pixbuf_image_format_infos = g_new (GOImageFormatInfo, pixbuf_format_nbr);
+		pixbuf_mimes = g_hash_table_new_full 
+			(g_str_hash, g_str_equal, g_free, g_free);
 
 		for (l = pixbuf_fmts, i = 1, format_info = pixbuf_image_format_infos; 
 		     l != NULL; 
@@ -166,6 +170,15 @@ go_image_build_pixbuf_format_infos (void)
 			format_info->has_pixbuf_saver = gdk_pixbuf_format_is_writable (fmt);
 			format_info->is_dpi_useful = FALSE;
 			format_info->alpha_support = FALSE;
+
+			mimes = gdk_pixbuf_format_get_mime_types (fmt);
+			for (j = 0; mimes[j]; j++) {
+				g_hash_table_insert
+					(pixbuf_mimes, 
+					 g_strdup((gpointer) mimes[j]),
+					 g_strdup((gpointer) format_info->name));
+			}
+			g_strfreev (mimes);
 		}
 	}
 
@@ -576,14 +589,18 @@ go_image_new_from_file (const char *filename, GError **error)
 {
 #ifdef GOFFICE_WITH_GTK
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (filename, error);
-	GOImage *image = g_object_new (GO_IMAGE_TYPE, "pixbuf", pixbuf, NULL);
-	g_object_unref (pixbuf);
-	image->target_cairo = FALSE;
-	return image;
+	if (pixbuf) {
+		GOImage *image = g_object_new (GO_IMAGE_TYPE,
+					       "pixbuf", pixbuf,
+					       NULL);
+		g_object_unref (pixbuf);
+		image->target_cairo = FALSE;
+		return image;
+	}
 #else
 	g_warning ("go_image_new_from_file not implemented!");
-	return NULL;
 #endif
+	return NULL;
 }
 
 guint8 *
