@@ -2,7 +2,7 @@
 /*
  * go-line.h : 
  *
- * Copyright (C) 2004 Emmanuel Pacaud (emmanuel.pacaud@univ-poitiers.fr)
+ * Copyright (C) 2004-2006 Emmanuel Pacaud (emmanuel.pacaud@univ-poitiers.fr)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -38,9 +38,22 @@ typedef enum {
 	GO_LINE_MAX
 } GOLineDashType;
 
+typedef enum {
+	GO_LINE_INTERPOLATION_LINEAR,
+	GO_LINE_INTERPOLATION_SPLINE,
+	GO_LINE_INTERPOLATION_STEP_START,
+	GO_LINE_INTERPOLATION_STEP_END,
+	GO_LINE_INTERPOLATION_STEP_CENTER_X,
+	GO_LINE_INTERPOLATION_STEP_CENTER_Y,
+	GO_LINE_INTERPOLATION_MAX
+} GOLineInterpolation;
+
 GOLineDashType	 go_line_dash_from_str		(char const *name);
 char const 	*go_line_dash_as_str		(GOLineDashType type);
 
+GOLineInterpolation	 go_line_interpolation_from_str		(char const *name);
+char const 	*go_line_interpolation_as_str		(GOLineInterpolation type);
+
 void 		 go_line_vpath_dash_free	(ArtVpathDash *dash);
 ArtVpathDash 	*go_line_get_vpath_dash 	(GOLineDashType type, double scale);
 
@@ -48,8 +61,6 @@ ArtVpath 	*go_line_clip_vpath		(ArtVpath const *path, GogViewAllocation const *b
 ArtVpath 	*go_line_dash_vpath 		(ArtVpath const *path, ArtVpathDash const *dash, 
 						 GogViewAllocation const *bbox);
 
-gpointer	 go_line_dash_selector		(GOLineDashType default_type);
-
 ArtBpath *go_line_build_bpath (double const *x, double const *y, int n);
 ArtVpath *go_line_build_vpath (double const *x, double const *y, int n);
 
