@@ -27,7 +27,7 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
-#include <goffice/data/go-data.h>
+#include <goffice/data/go-data-simple.h>
 #include <goffice/utils/go-math.h>
 #include <goffice/app/module-plugin-defs.h>
 
