@@ -40,6 +40,7 @@
 #include <glib/gi18n-lib.h>
 #include <gsf/gsf-impl-utils.h>
 #include <math.h>
+#include <string.h>
 
 typedef struct {
 	GogPlotClass	base;
@@ -244,7 +245,8 @@ enum {
 	GOG_XY_PROP_0,
 	GOG_XY_PROP_DEFAULT_STYLE_HAS_MARKERS,
 	GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES,
-	GOG_XY_PROP_USE_SPLINES
+	GOG_XY_PROP_USE_SPLINES,
+	GOG_XY_PROP_INTERPOLATION,
 };
 
 static GogObjectClass *xy_parent_klass;
@@ -269,12 +271,19 @@ gog_xy_set_property (GObject *obj, guint param_id,
 	case GOG_XY_PROP_DEFAULT_STYLE_HAS_MARKERS:
 		xy->default_style_has_markers = g_value_get_boolean (value);
 		break;
-	case GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES:
+	case GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES: {
 		xy->default_style_has_lines = g_value_get_boolean (value);
 		break;
+	}
 	case GOG_XY_PROP_USE_SPLINES:
-		xy->use_splines = g_value_get_boolean (value);
+		if (g_value_get_boolean (value))
+			xy->interpolation = GO_LINE_INTERPOLATION_SPLINE;
 		break;
+	case GOG_XY_PROP_INTERPOLATION: {
+		char const *s = g_value_get_string (value);
+		xy->interpolation = go_line_interpolation_from_str (s);;
+		break;
+	}
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -292,7 +301,10 @@ gog_xy_get_property (GObject *obj, guint param_id,
 		g_value_set_boolean (value, xy->default_style_has_lines);
 		break;
 	case GOG_XY_PROP_USE_SPLINES:
-		g_value_set_boolean (value, xy->use_splines);
+		g_value_set_boolean (value, xy->interpolation == GO_LINE_INTERPOLATION_SPLINE);
+		break;
+	case GOG_XY_PROP_INTERPOLATION:
+		g_value_set_string (value, go_line_interpolation_as_str (xy->interpolation));
 		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
@@ -322,7 +334,10 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 		g_param_spec_boolean ("use-splines", NULL,
 			"Should the plot use splines instead of linear interpolation",
 			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
-
+	g_object_class_install_property (gobject_klass, GOG_XY_PROP_INTERPOLATION,
+		g_param_spec_string  ("interpolation", NULL,
+			_("Interpolation type (none, linear, spline or step) with variant, if any."),
+            "none", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	gog_klass->type_name	= gog_xy_plot_type_name;
 
 	{
@@ -343,7 +358,7 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 		};
 		plot_klass->desc.series.dim = dimensions;
 		plot_klass->desc.series.num_dim = G_N_ELEMENTS (dimensions);
-		plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER;
+		plot_klass->desc.series.style_fields = GOG_STYLE_LINE | GOG_STYLE_MARKER | GOG_STYLE_INTERPOLATION;
 	}
 }
 
@@ -551,7 +566,6 @@ bubble_draw_circle (GogView *view, double x, double y, double radius)
 	path[0].x = path[MAX_ARC_SEGMENTS].x = x + radius;
 	path[0].y = path[MAX_ARC_SEGMENTS].y = y;
 	path[0].code = ART_MOVETO;
-#warning what about small bubbles. With a very small radius, libart emits lot of warnings.
 	if (radius < 1.) radius = 1.;
 	for (i = 1, theta = dt; i < MAX_ARC_SEGMENTS; i++, theta += dt) {
 		path[i].x = x + radius * cos (theta);
@@ -737,16 +751,103 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 					gog_axis_map_to_view (y_map, y_vals[i]):
 					go_nan;
 			}
-			if (GOG_XY_PLOT (view->model)->use_splines) {
-				ArtBpath *path;
-				path = go_line_build_bpath (x_splines, y_splines, n);
-				gog_renderer_draw_bezier_path (view->renderer, path);
-				art_free (path);
-			} else {
+			switch (style->interpolation.type) {
+			case GO_LINE_INTERPOLATION_LINEAR: {
 				ArtVpath *path;
 				path = go_line_build_vpath (x_splines, y_splines, n);
 				gog_renderer_draw_path (view->renderer, path);
 				art_free (path);
+				break;
+			}
+			case GO_LINE_INTERPOLATION_SPLINE: {
+				ArtBpath *path;
+				path = go_line_build_bpath (x_splines, y_splines, n);
+				gog_renderer_draw_bezier_path (view->renderer, path);
+				art_free (path);
+				break;
+			}
+			case GO_LINE_INTERPOLATION_STEP_START:
+			case GO_LINE_INTERPOLATION_STEP_END:
+			case GO_LINE_INTERPOLATION_STEP_CENTER_X:
+			case GO_LINE_INTERPOLATION_STEP_CENTER_Y: {
+				unsigned i, j;
+				ArtVpath *path;
+				gboolean b = FALSE;
+				i = 0;
+				while ((!go_finite (x_splines[i]) || !go_finite (y_splines[i]))  && n < i)
+					i++;
+				if (n == i)
+					break;
+				path = art_new (ArtVpath,
+					((style->interpolation.type <= GO_LINE_INTERPOLATION_STEP_END)?
+					2 * n + 1: 3 * n + 1));
+				path[0].code = ART_MOVETO;
+				path[0].x = x_splines[i];
+				path[0].y = y_splines[i++];
+				for (j = 1; i < n; i++) {
+					while ((!go_finite (x_splines[i]) ||
+							!go_finite (y_splines[i]))  && i < n) {
+						i++;
+						b = TRUE;
+					}
+					if (n == i)
+						break;
+					if (b) {
+						path[j].code = ART_MOVETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						b = FALSE;
+					} else switch (style->interpolation.type) {
+					case GO_LINE_INTERPOLATION_STEP_START:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i-1];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_END:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i-1];
+						path[j++].y = y_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_CENTER_X:
+						path[j].code = ART_LINETO;
+						path[j].x = path[j+1].x = (x_splines[i-1] + x_splines[i]) / 2.;
+						path[j++].y = y_splines[i-1];
+						path[j].code = ART_LINETO;
+						path[j++].y = y_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					case GO_LINE_INTERPOLATION_STEP_CENTER_Y:
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i-1];
+						path[j++].y = (y_splines[i-1] + y_splines[i]) / 2.;
+						path[j].code = ART_LINETO;
+						path[j].y = path[j-1].y;
+						path[j++].x = x_splines[i];
+						path[j].code = ART_LINETO;
+						path[j].x = x_splines[i];
+						path[j++].y = y_splines[i];
+						break;
+					default:
+						g_assert_not_reached ();
+						break;
+					}
+				}
+				path[j].code = ART_END;
+				gog_renderer_draw_sharp_path (view->renderer, path);
+				art_free (path);
+				break;
+			}
+			default:
+				g_assert_not_reached ();
+				break;
 			}
 			g_free (x_splines);
 			g_free (y_splines);
@@ -1077,6 +1178,9 @@ gog_xy_series_init_style (GogStyledObject *gso, GogStyle *style)
 	if (!plot->default_style_has_lines &&
 	    style->line.auto_dash)
 		style->line.dash_type = GO_LINE_NONE;
+
+	if (style->interpolation.auto_type)
+		style->interpolation.type = plot->interpolation;
 }
 
 static void
@@ -1175,6 +1279,8 @@ gog_xy_series_populate_editor (GogObject *obj,
 	GladeXML *gui = go_libglade_new (path, "gog-xy-series-prefs", GETTEXT_PACKAGE, cc);
 
 	g_free (path);
+	(GOG_OBJECT_CLASS(series_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
+
 	if (gui != NULL) {
 		w = glade_xml_get_widget (gui, "invalid-as-zero");
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
@@ -1188,8 +1294,6 @@ gog_xy_series_populate_editor (GogObject *obj,
 		gog_editor_add_page (editor, w, _("Details"));
 	}
 
-	(GOG_OBJECT_CLASS(series_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
-
 	w = gog_error_bar_prefs (GOG_SERIES (obj), "x-errors", TRUE, dalloc, cc);
 	gog_editor_add_page (editor, w, _("X error bars"));
 	w = gog_error_bar_prefs (GOG_SERIES (obj), "y-errors", FALSE, dalloc, cc);
